.setcpu	"6502"

; Registers
; =========

PPU_CTRL	= $2000
PPU_MASK	= $2001
PPU_STATUS	= $2002
PPU_SCROLL	= $2005
PPU_ADDR	= $2006
PPU_DATA	= $2007

OAM_ADDR	= $2003
OAM_DATA	= $2004
OAM_DMA		= $4014

APU_SQ0_VOL	= $4000
APU_SQ0_SWEEP	= $4001
APU_SQ0_LO	= $4002
APU_SQ0_HI	= $4003
APU_SQ1_VOL	= $4004
APU_SQ1_SWEEP	= $4005
APU_SQ1_LO	= $4006
APU_SQ1_HI	= $4007
APU_TRI_LINEAR	= $4008
APU_TRI_LO	= $400A
APU_TRI_HI	= $400B
APU_NOISE_VOL	= $400C
APU_NOISE_LO	= $400E
APU_NOISE_HI	= $400F
APU_DMC_FREQ	= $4010
APU_DMC_RAW	= $4011
APU_DMC_START	= $4012
APU_DMC_LEN	= $4013
APU_CTRL	= $4015
APU_FRAME	= $4017

JOY0	= $4016
JOY1	= $4017


; Constants
; =========

VRAM_PT0	= $0000
VRAM_PT1	= $1000
VRAM_NT0	= $2000
VRAM_NT1	= $2400
VRAM_NT2	= $2800
VRAM_NT3	= $2C00
VRAM_ATTR0	= $23C0
VRAM_ATTR1	= $27C0
VRAM_ATTR2	= $2BC0
VRAM_ATTR3	= $2FC0
VRAM_PLT_BG	= $3F00
VRAM_PLT_BG0	= $3F00
VRAM_PLT_BG1	= $3F04
VRAM_PLT_BG2	= $3F08
VRAM_PLT_BG3	= $3F0C
VRAM_PLT_SPR	= $3F10
VRAM_PLT_SPR0	= $3F10
VRAM_PLT_SPR1	= $3F14
VRAM_PLT_SPR2	= $3F18
VRAM_PLT_SPR3	= $3F1C

COLOR_BLACK	= $0F
COLOR_WHITE	= $30

PPU_CTRL_NT		= %00000011
PPU_CTRL_NT0		= %00000000
PPU_CTRL_NT1		= %00000001
PPU_CTRL_NT2		= %00000010
PPU_CTRL_NT3		= %00000011
PPU_CTRL_INC32		= %00000100
PPU_CTRL_SPR_PT1	= %00001000
PPU_CTRL_BG_PT1		= %00010000
PPU_CTRL_SPR_8x16	= %00100000
PPU_CTRL_NMI		= %10000000

PPU_MASK_GRAY		= %00000001
PPU_MASK_BG_NOCLIP	= %00000010
PPU_MASK_SPR_NOCLIP	= %00000100
PPU_MASK_BG_ON		= %00001000
PPU_MASK_SPR_ON		= %00010000
PPU_MASK_INT_RED	= %00100000
PPU_MASK_INT_GREEN	= %01000000
PPU_MASK_INT_BLUE	= %10000000

PPU_STATUS_SPR_OVERFLOW	= %00100000
PPU_STATUS_SPR_HIT0	= %01000000
PPU_STATUS_VBL		= %10000000

SPR_ATTR_PLT		= %00000011
SPR_ATTR_PLT0		= %00000000
SPR_ATTR_PLT1		= %00000001
SPR_ATTR_PLT2		= %00000010
SPR_ATTR_PLT3		= %00000011
SPR_ATTR_BEHIND		= %00100000
SPR_ATTR_FLIP_HORIZ	= %01000000
SPR_ATTR_FLIP_VERT	= %10000000

APU_CTRL_SQ0_ON		= %00000001
APU_CTRL_SQ1_ON		= %00000010
APU_CTRL_TRI_ON		= %00000100
APU_CTRL_NOISE_ON	= %00001000
APU_CTRL_DMC_ON		= %00010000
APU_CTRL_FRAME_IRQ	= %01000000
APU_CTRL_DMC_IRQ	= %10000000

APU_FRAME_NOIRQ = %01000000
APU_FRAME_5STEP	= %10000000

JOY0_BTN	= %00000001
JOY0_EX		= %00000010
JOY0_MIC	= %00000100
JOY1_BTN	= %00000001

BTN_A		= %10000000
BTN_B		= %01000000
BTN_SELECT	= %00100000
BTN_START	= %00010000
BTN_UP		= %00001000
BTN_DOWN	= %00000100
BTN_LEFT	= %00000010
BTN_RIGHT	= %00000001


; Macros
; ======

.macro	PPU_WAIT_VBL
	.local	loop
loop:	bit	PPU_STATUS
	bpl	loop
.endmacro

.macro	PPU_SET_SCROLL	xx, yy
	lda	#xx
	sta	PPU_SCROLL
	lda	#yy
	sta	PPU_SCROLL
.endmacro

.macro	PPU_SET_ADDR	addr
	lda	#>addr
	sta	PPU_ADDR
	lda	#<addr
	sta	PPU_ADDR
.endmacro

.define	PPU_SET_ADDR_NT(nt, xx, yy)	PPU_SET_ADDR($2000 + $0400*nt + $20*yy + xx)

.macro	JOY_UPDATE
	lda	#1
	sta	JOY0
	lda	#0
	sta	JOY0
.endmacro


; Generating header and vectors
; =============================

.segment	"HEADER"
	.byte	$4E, $45, $53, $1A	; "NES\x1A"
	.byte	CART_PRG_COUNT
	.byte	CART_CHR_COUNT
	.byte	CART_MIRROR_VERT | (CART_SRAM<<1) | (CART_MIRROR_FOUR<<3) | ((CART_MAPPER&$0F)<<4)
	.byte	CART_MAPPER&$F0
	.byte	$00, $00, $00, $00, $00, $00, $00, $00

.segment	"VECTORS"
	.word	NMI
	.word	RESET
	.word	IRQ
